@@ -6,21 +6,22 @@
 
 use anyhow::{anyhow, Result};
 
+use sequoia_openpgp::cert::amalgamation::key::ValidErasedKeyAmalgamation;
+use sequoia_openpgp::packet::key::SecretParts;
 use sequoia_openpgp::policy::Policy;
 use sequoia_openpgp::Cert;
 
 use openpgp_card::algorithm::{Algo, AlgoInfo};
 use openpgp_card::card_do::{
     ApplicationIdentifier, ApplicationRelatedData, CardholderRelatedData,
-    ExCapFeatures, ExtendedCapabilities, ExtendedLengthInfo, Fingerprint,
-    HistoricalBytes, KeyGenerationTime, PWStatusBytes,
-    SecuritySupportTemplate, Sex,
+    ExtendedCapabilities, ExtendedLengthInfo, Fingerprint, HistoricalBytes,
+    KeyGenerationTime, PWStatusBytes, SecuritySupportTemplate, Sex,
 };
-use openpgp_card::crypto_data::CardUploadableKey;
 use openpgp_card::{CardApp, CardClientBox, Error, KeySet, KeyType, Response};
 
 use crate::decryptor::CardDecryptor;
 use crate::signer::CardSigner;
+use crate::util::vka_as_uploadable_key;
 
 /// Representation of an opened OpenPGP card in its base state (i.e. no
 /// passwords have been verified, default authorization applies).
@@ -103,6 +104,32 @@ impl Open {
     pub fn check_admin_verified(&mut self) -> Result<Response, Error> {
         self.card_app.check_pw3()
     }
+
+    pub fn change_user_pin(
+        &mut self,
+        old: &str,
+        new: &str,
+    ) -> Result<Response, Error> {
+        self.card_app.change_pw1(old, new)
+    }
+
+    pub fn reset_user_pin(
+        &mut self,
+        rst: &str,
+        new: &str,
+    ) -> Result<Response, Error> {
+        self.card_app
+            .reset_retry_counter_pw1(new.into(), Some(rst.into()))
+    }
+
+    pub fn change_admin_pin(
+        &mut self,
+        old: &str,
+        new: &str,
+    ) -> Result<Response, Error> {
+        self.card_app.change_pw3(old, new)
+    }
+
     /// Get a view of the card authenticated for "User" commands.
     pub fn user_card(&mut self) -> Option<User> {
         if self.pw1 {
@@ -242,7 +269,7 @@ impl Open {
         // The DO "Algorithm Information" (Tag FA) shall be present if
         // Algorithm attributes can be changed
         let ec = self.extended_capabilities()?;
-        if !ec.features().contains(&ExCapFeatures::AlgoAttrsChangeable) {
+        if !ec.algo_attrs_changeable() {
             // Algorithm attributes can not be changed,
             // list_supported_algo is not supported
             return Ok(None);
@@ -333,18 +360,40 @@ impl Admin<'_> {
         // Check for max len
         let ec = self.oc.extended_capabilities()?;
 
-        if url.len() < ec.max_len_special_do() as usize {
+        if ec.max_len_special_do() == None
+            || url.len() <= ec.max_len_special_do().unwrap() as usize
+        {
+            // If we don't know the max length for URL ("special DO"),
+            // or if it's within the acceptable length:
+            // send the url update to the card.
+
             self.oc.card_app.set_url(url)
         } else {
             Err(anyhow!("URL too long").into())
         }
     }
 
+    pub fn set_resetting_code(
+        &mut self,
+        pin: &str,
+    ) -> Result<Response, Error> {
+        self.oc.card_app.set_resetting_code(pin.into())
+    }
+
+    pub fn reset_user_pin(&mut self, new: &str) -> Result<Response, Error> {
+        self.oc.card_app.reset_retry_counter_pw1(new.into(), None)
+    }
+
+    /// Upload a ValidErasedKeyAmalgamation to the card as a specific KeyType.
+    ///
+    /// (The caller needs to make sure that `vka` is suitable as `key_type`)
     pub fn upload_key(
         &mut self,
-        key: Box<dyn CardUploadableKey>,
+        vka: ValidErasedKeyAmalgamation<SecretParts>,
         key_type: KeyType,
+        password: Option<String>,
     ) -> Result<(), Error> {
+        let key = vka_as_uploadable_key(vka, password);
         self.oc.card_app.key_import(key, key_type)
     }
 }
