@@ -18,19 +18,19 @@ use openpgp_card_sequoia::sq_util::{decryption_helper, sign_helper};
 
 // Filename of test key and test message to use
 
-// const TEST_KEY_PATH: &str = "../example/test4k.sec";
-// const TEST_ENC_MSG: &str = "../example/encrypted_to_rsa4k.asc";
+// const TEST_KEY_PATH: &str = "example/test4k.sec";
+// const TEST_ENC_MSG: &str = "example/encrypted_to_rsa4k.asc";
 
-// const TEST_KEY_PATH: &str = "../example/nist521.sec";
-// const TEST_ENC_MSG: &str = "../example/encrypted_to_nist521.asc";
+// const TEST_KEY_PATH: &str = "example/nist521.sec";
+// const TEST_ENC_MSG: &str = "example/encrypted_to_nist521.asc";
 
-const TEST_KEY_PATH: &str = "../example/test25519.sec";
-const TEST_ENC_MSG: &str = "../example/encrypted_to_25519.asc";
+const TEST_KEY_PATH: &str = "example/test25519.sec";
+const TEST_ENC_MSG: &str = "example/encrypted_to_25519.asc";
 
 fn main() -> Result<(), Box<dyn Error>> {
     env_logger::init();
 
-    // Ident of an OpenPGP Card to use for these tests
+    // Ident of an OpenPGP card to use for these tests
     let test_card_ident = env::var("TEST_CARD_IDENT");
 
     if let Ok(test_card_ident) = test_card_ident {
@@ -115,30 +115,34 @@ fn main() -> Result<(), Box<dyn Error>> {
         println!("set url {:x?}", res);
 
         let cert = Cert::from_file(TEST_KEY_PATH)?;
+        let p = StandardPolicy::new();
 
-        println!("Upload decryption key");
-        openpgp_card_sequoia::util::upload_from_cert_yolo(
-            &mut admin,
-            &cert,
-            KeyType::Decryption,
-            None,
-        )?;
-
-        println!("Upload signing key");
-        openpgp_card_sequoia::util::upload_from_cert_yolo(
-            &mut admin,
+        if let Some(vka) = openpgp_card_sequoia::sq_util::get_subkey(
             &cert,
+            &p,
             KeyType::Signing,
-            None,
-        )?;
+        )? {
+            println!("Upload signing key");
+            admin.upload_key(vka, KeyType::Signing, None)?;
+        }
 
-        // FIXME: Test keys have no authentication subkey
-        // openpgp_card_sequoia::util::upload_from_cert_yolo(
-        //     &mut admin,
-        //     &cert,
-        //     KeyType::Authentication,
-        //     None,
-        // )?;
+        if let Some(vka) = openpgp_card_sequoia::sq_util::get_subkey(
+            &cert,
+            &p,
+            KeyType::Decryption,
+        )? {
+            println!("Upload decryption key");
+            admin.upload_key(vka, KeyType::Decryption, None)?;
+        }
+
+        if let Some(vka) = openpgp_card_sequoia::sq_util::get_subkey(
+            &cert,
+            &p,
+            KeyType::Authentication,
+        )? {
+            println!("Upload auth key");
+            admin.upload_key(vka, KeyType::Authentication, None)?;
+        }
 
         println!();
 
