@@ -8,7 +8,7 @@ use std::convert::TryInto;
 use std::io;
 use std::time::SystemTime;
 
-use anyhow::{anyhow, Context, Result};
+use anyhow::{Context, Result};
 
 use openpgp::armor;
 use openpgp::cert::amalgamation::key::ValidErasedKeyAmalgamation;
@@ -19,7 +19,7 @@ use openpgp::packet::{
     Key, UserID,
 };
 use openpgp::parse::{stream::DecryptorBuilder, Parse};
-use openpgp::policy::StandardPolicy;
+use openpgp::policy::Policy;
 use openpgp::serialize::stream::{Message, Signer};
 use openpgp::types::{KeyFlags, PublicKeyAlgorithm, SignatureType, Timestamp};
 use openpgp::{Cert, Packet};
@@ -30,7 +30,6 @@ use openpgp_card::card_do::{Fingerprint, KeyGenerationTime};
 use openpgp_card::crypto_data::{CardUploadableKey, PublicKeyMaterial};
 use openpgp_card::{CardApp, Error, KeyType};
 
-use crate::card::Admin;
 use crate::privkey::SequoiaKey;
 use crate::signer::CardSigner;
 use crate::{decryptor, signer, PublicKey};
@@ -38,6 +37,8 @@ use crate::{decryptor, signer, PublicKey};
 /// Create a Cert from the three subkeys on a card.
 /// (Calling this multiple times will result in different Certs!)
 ///
+/// FIXME: contains hardcoded default passwords!
+///
 /// FIXME: make dec/auth keys optional
 ///
 /// FIXME: accept optional metadata for user_id(s)?
@@ -145,15 +146,6 @@ pub fn make_cert(
     Cert::try_from(pp)
 }
 
-/// Helper fn: get a CardUploadableKey for a ValidErasedKeyAmalgamation
-pub fn vka_as_uploadable_key(
-    vka: ValidErasedKeyAmalgamation<SecretParts>,
-    password: Option<String>,
-) -> Box<dyn CardUploadableKey> {
-    let sqk = SequoiaKey::new(vka, password);
-    Box::new(sqk)
-}
-
 /// Helper fn: get a Sequoia PublicKey from an openpgp-card PublicKeyMaterial
 pub fn public_key_material_to_key(
     pkm: &PublicKeyMaterial,
@@ -257,56 +249,13 @@ pub fn public_to_fingerprint(
     fp.as_bytes().try_into()
 }
 
-/// FIXME: this is used in main.rs for testing, but should probably not exist.
-///
-/// Convenience fn to select and upload a (sub)key from a Cert, as a given
-/// KeyType. If multiple suitable (sub)keys are found, the first one is
-/// used.
-///
-/// (sub)keys for upload should probably always be explicitly picked by
-/// client software.
-pub fn upload_from_cert_yolo(
-    oca: &mut Admin,
-    cert: &Cert,
-    key_type: KeyType,
-    password: Option<String>,
-) -> Result<(), Box<dyn std::error::Error>> {
-    let policy = StandardPolicy::new();
-
-    // Find all suitable (sub)keys for key_type.
-    let mut valid_ka = cert
-        .keys()
-        .with_policy(&policy, None)
-        .secret()
-        .alive()
-        .revoked(false);
-    valid_ka = match key_type {
-        KeyType::Decryption => valid_ka.for_storage_encryption(),
-        KeyType::Signing => valid_ka.for_signing(),
-        KeyType::Authentication => valid_ka.for_authentication(),
-        _ => return Err(anyhow!("Unexpected KeyType").into()),
-    };
-
-    // FIXME: for now, we just pick the first (sub)key from the list
-    if let Some(vka) = valid_ka.next() {
-        upload_key(oca, vka, key_type, password).map_err(|e| e.into())
-    } else {
-        Err(anyhow!("No suitable (sub)key found").into())
-    }
-}
-
-/// Upload a ValidErasedKeyAmalgamation to the card as a specific KeyType.
-///
-/// The caller needs to make sure that `vka` is suitable for `key_type`.
-pub fn upload_key(
-    oca: &mut Admin,
+/// Helper fn: get a CardUploadableKey for a ValidErasedKeyAmalgamation
+pub fn vka_as_uploadable_key(
     vka: ValidErasedKeyAmalgamation<SecretParts>,
-    key_type: KeyType,
     password: Option<String>,
-) -> Result<(), Error> {
+) -> Box<dyn CardUploadableKey> {
     let sqk = SequoiaKey::new(vka, password);
-
-    oca.upload_key(Box::new(sqk), key_type)
+    Box::new(sqk)
 }
 
 /// FIXME: this fn is used in card_functionality, but should be removed
@@ -314,11 +263,11 @@ pub fn sign(
     ca: &mut CardApp,
     cert: &Cert,
     input: &mut dyn io::Read,
+    p: &dyn Policy,
 ) -> Result<String> {
     let mut armorer = armor::Writer::new(vec![], armor::Kind::Signature)?;
     {
-        let p = StandardPolicy::new();
-        let s = signer::CardSigner::new(ca, cert, &p)?;
+        let s = signer::CardSigner::new(ca, cert, p)?;
 
         let message = Message::new(&mut armorer);
         let mut message = Signer::new(message, s).detached().build()?;
@@ -339,16 +288,16 @@ pub fn decrypt(
     ca: &mut CardApp,
     cert: &Cert,
     msg: Vec<u8>,
+    p: &dyn Policy,
 ) -> Result<Vec<u8>> {
     let mut decrypted = Vec::new();
     {
         let reader = io::BufReader::new(&msg[..]);
 
-        let p = StandardPolicy::new();
-        let d = decryptor::CardDecryptor::new(ca, cert, &p)?;
+        let d = decryptor::CardDecryptor::new(ca, cert, p)?;
 
         let db = DecryptorBuilder::from_reader(reader)?;
-        let mut decryptor = db.with_policy(&p, None, d)?;
+        let mut decryptor = db.with_policy(p, None, d)?;
 
         // Read all data from decryptor and store in decrypted
         io::copy(&mut decryptor, &mut decrypted)?;
